package com.servicemall.loyalty.micro;

import java.util.List;
import java.util.Map;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.*;

import springfox.documentation.annotations.ApiIgnore;
import team.bangbang.common.data.Pagination;
import team.bangbang.common.data.response.ResponseBase;
import team.bangbang.common.data.response.DataResponse;
import team.bangbang.common.log.OperationLog;
import team.bangbang.spring.parameter.EntityParam;

import com.servicemall.loyalty.data.Plan;
import com.servicemall.loyalty.service.PlanService;

/**
 * 礼遇计划 - 微服务
 *
 * 对应Feign：com.servicemall.loyalty.feign.PlanFeign
 *
 * @author 帮帮组
 * @version 1.0  2020-12-08
 */
@Api(tags = {"loyalty"})
@ApiResponses(value={@ApiResponse(code=200, message="请求的服务不存在")})
@RestController
@CrossOrigin(allowCredentials="true", allowedHeaders="*", origins="*", maxAge=3600)
@RequestMapping("/microservice/loyalty/plan")
public final class PlanMicro {
	/**************************************************************************
	 * ！！除非设计、指导人员有特别说明，否则此处不得随意增加、修改、删除！！
	 * －－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－
	 *
	 *************************************************************************/

	private static final Logger logger = LoggerFactory.getLogger(PlanMicro.class);

	/**
	 * 插入一条礼遇计划
	 *
	 * @param plan 插入的数据，不能为null
     * @param request HTTP请求
	 *
	 * @return 1：成功 其它：失败
	 */
	@PostMapping("/insert")
	@ApiIgnore
	public ResponseBase insert(@RequestBody Plan plan, HttpServletRequest request) {
		// 是否存在重复记录？
		if(exist(plan)) {
			return ResponseBase.DATA_DUPLICATE;
		}

		int n = PlanService.insert(plan);

		if(n != 1) {
			return ResponseBase.ERROR_LOGIC;
		}

        // 记录日志
        OperationLog log = new OperationLog();
        log.setType("新增礼遇计划");
        log.setBizData(plan);
        request.setAttribute("log", log);

		DataResponse<Plan> result = new DataResponse<Plan>();
		result.setMessage("成功");
		result.setData(plan);

		return result;
	}

	/**
	 * 删除礼遇计划
	 *
	 * @param plan 删除条件，必须包含关键字段
     * @param request HTTP请求
	 *
	 * @return 成功删除的记录数量
	 */
	@ApiIgnore
	@PostMapping("/delete")
	public ResponseBase delete(@RequestBody Plan plan, HttpServletRequest request) {

		if(plan.getId() == null || plan.getId() == 0L) {
			return ResponseBase.DATA_NOT_FOUND;
		}

        // 获取待删除的对象，用于日志记录
        plan = PlanService.getObject(plan, null);
        if(plan == null) {
            return ResponseBase.DATA_NOT_FOUND;
        }

		// 限定条件
		Plan where = new Plan();
		where.setId(plan.getId());

		int result = PlanService.delete(where, null);

		if(result != 1) {
			return ResponseBase.DATA_NOT_FOUND;
		}

        // 记录日志
        OperationLog log = new OperationLog();
        log.setType("删除礼遇计划");
        log.setBizData(where);
        request.setAttribute("log", log);

		return ResponseBase.SUCCESS;
	}

	/**
	 * 修改礼遇计划
	 *
	 * @param plan 更新条件，不能为null
     * @param request HTTP请求
	 *
	 * @return 成功修改的记录数量
	 */
	@ApiIgnore
	@PostMapping("/update")
	public ResponseBase update(@RequestBody Plan plan, HttpServletRequest request) {
		// 为防止更新意外，必须传入id才能更新
		Long id = plan.getId();

		if(id == null || id == 0L) {
			return ResponseBase.DATA_NOT_FOUND;
		}

		// 是否存在重复记录？
		if(exist(plan)) {
			return ResponseBase.DATA_DUPLICATE;
		}

		// 第1个参数plan，取关键字段plan.id为条件
		// 第3个参数plan，取plan内关键字段以外其它属性数据
		int n = PlanService.update(plan, null, plan);

		if(n != 1) {
			return ResponseBase.DATA_NOT_FOUND;
		}

        // 记录日志
        OperationLog log = new OperationLog();
        log.setType("修改礼遇计划");
        log.setBizData(plan);
        request.setAttribute("log", log);

		// 重新查询，返回到前端
		plan = PlanService.getObject(plan, null);
		DataResponse<Plan> result = new DataResponse<Plan>();
		result.setData(plan);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 查询一条礼遇计划，并转化为相应的POJO对象
	 *
	 * @param plan 查询条件，不能为null
	 *
	 * @return 返回结果记录，并转化为相应的POJO对象
	 */
	@ApiIgnore
	@PostMapping("/get")
	public ResponseBase get(@RequestBody Plan plan) {

		if(plan.getId() == null || plan.getId() == 0L) {
			return ResponseBase.DATA_NOT_FOUND;
		}

		// 查询
		plan = PlanService.getObject(plan, null);
		DataResponse<Plan> result = new DataResponse<Plan>();
		result.setData(plan);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 查询多条礼遇计划，并转化为相应的POJO对象列表
	 *
	 * @param plan 更新条件，不能为null
	 *
	 * @return 返回结果记录，并转化为相应的POJO对象列表
	 */
	@ApiOperation(value = "首页API接口", notes = "首页API接口", httpMethod = "POST")
	@PostMapping("/list")
	public ResponseBase list(@RequestBody Plan plan, Pagination pagination) {

		// 附加限定条件，这里可以修改附加限定条件
		String appendix = null;

		List<Plan> planList = PlanService.list(plan, appendix, pagination);

		// 统计符合条件的结果记录数量
		int recordCount = PlanService.count(plan, appendix);
		pagination.setRecordCount(recordCount);

		DataResponse<Map<String, Object>> result = new DataResponse<Map<String, Object>>();
		Map<String, Object> datas = new HashMap<String, Object>();
		datas.put("list", planList);
		datas.put("pagination", pagination);

		result.setData(datas);

		return result;
	}

	/**
	 * 获得符合条件的礼遇计划数量
	 *
	 * @param plan 查询条件，不能为null
	 *
	 * @return 返回记录数量
	 */
	@ApiIgnore
	@PostMapping("/count")
	public ResponseBase count(@RequestBody Plan plan) {
		// 附加限定条件，这里可以修改附加限定条件
		String appendix = null;

		int n = PlanService.count(plan, appendix);
		DataResponse<Integer> result = new DataResponse<Integer>();

		result.setData(n);
		result.setMessage("成功");
		return result;
	}

	/**
	 * 用在新增、修改时检查数据库中是否存在重复记录（请务必保留该方法）
	 *
	 * @param temp
	 *            将要修改的礼遇计划POJO
	 * @return true：已经存在 false：不存在
	 */
	private boolean exist(Plan temp) {
		// 检查修改的礼遇计划是否有重复记录
		Plan form = new Plan();

		// 关键字限定条件

		String str = (temp.getId() == null ? null : "id != " + temp.getId());

		// 其它信息限定条件

		// 礼遇名称
		form.setLoyaltyName(temp.getLoyaltyName());

		// 购买数量限制
		form.setBuyAmountLimit(temp.getBuyAmountLimit());

		// 厂家价格,单位 元
		form.setFactoryGuidePrice(temp.getFactoryGuidePrice());

		// 售价类型{1:统一价格 2:车型动态价格}
		form.setPriceTypeFlag(temp.getPriceTypeFlag());

		// 统一售价,单位 元
		form.setUniformPrice(temp.getUniformPrice());

		// 是否显示在精选位置{0:否 1:是}
		form.setSelectedStatus(temp.getSelectedStatus());

		// 核销时是否绑定车辆使用{0:否 1:是}
		form.setBindingUseStatus(temp.getBindingUseStatus());

		// 付款方式{1:定金 2:全款}
		form.setPayTypeFlag(temp.getPayTypeFlag());

		// 定金比例
		form.setDepositRatio(temp.getDepositRatio());

		// 定金金额，单位元
		form.setDepositPrice(temp.getDepositPrice());

		// 适用车龄
		form.setApplyOld(temp.getApplyOld());

		// 是否加入购物车{0:否 1:是}
		form.setCartEnableStatus(temp.getCartEnableStatus());

		// 计划描述
		form.setDescription(temp.getDescription());

		// 购买限制说明
		form.setBuyLimitDescription(temp.getBuyLimitDescription());

		// 购买起始时间
		form.setBuyStartTime(temp.getBuyStartTime());

		// 购买结束时间
		form.setBuyEndTime(temp.getBuyEndTime());

		// 创建时间
		form.setCreateAt(temp.getCreateAt());

		// 更新时间
		form.setUpdateAt(temp.getUpdateAt());

		// 计划图片
		form.setPlanImage(temp.getPlanImage());

		// 活动轮播图
		form.setActivityImage(temp.getActivityImage());

		// 创建人编号
		form.setCreatorId(temp.getCreatorId());

		// 修改人编号
		form.setUpdatorId(temp.getUpdatorId());

		// 删除状态{0:否 1:是}
		form.setDelStatus(temp.getDelStatus());

		// 生效状态{1:待生效 2:生效中 3:已暂停 4:已结束}
		form.setActiveStatus(temp.getActiveStatus());

		return (PlanService.getObject(form, str) != null);
	}
}
